import "reflect-metadata";
import {ContainerModule, interfaces} from "inversify";
import {PersistenceService} from "./persistence/PersistenceService";
import {FrameworkBootstrapper} from "./bootstrapper/FrameworkBootstrapper";
import {LinkService} from "./link/LinkService";
import {ConfigurationService} from "./configuration/ConfigurationService";
import {PackageService} from "./node/PackageService";
import {NpmService} from "./node/NpmService";
import {VersionService} from "./version/VersionService";

export class FrameworkModule extends ContainerModule {

    constructor() {
        super(FrameworkModule.initializeBindings);
    }

    private static initializeBindings(bind: interfaces.Bind): void {
        bind<FrameworkBootstrapper>("FrameworkBootstrapper").to(FrameworkBootstrapper);
        FrameworkModule.initializeConfigurationBindings(bind);
        FrameworkModule.initializeNodeBindings(bind);
        FrameworkModule.initializePersistenceBindings(bind);
        FrameworkModule.initializeLinkBindings(bind);
        FrameworkModule.initializeVersionBindings(bind);
    }

    private static initializeConfigurationBindings(bind: interfaces.Bind): void {
        bind<ConfigurationService>("ConfigurationService").to(ConfigurationService);
    }

    private static initializeNodeBindings(bind: interfaces.Bind): void {
        bind<PackageService>("PackageService").to(PackageService);
        bind<NpmService>("NpmService").to(NpmService);
    }

    private static initializePersistenceBindings(bind: interfaces.Bind): void {
        bind<PersistenceService>("PersistenceService").to(PersistenceService);
    }

    private static initializeLinkBindings(bind: interfaces.Bind): void {
        bind<LinkService>("LinkService").to(LinkService);
    }

    private static initializeVersionBindings(bind: interfaces.Bind): void {
        bind<VersionService>("VersionService").to(VersionService);
    }
}