import {Container, interfaces} from "inversify";
import {FrameworkModule} from "../FrameworkModule";
import {LoggingModule} from "../logging/LoggingModule";
import {LogLevel} from "../logging/LogLevel";
import {ILoggingPipeline} from "../logging/ILoggingPipeline";
import BindingScope = interfaces.BindingScope;

export class FrameworkBootstrapper {
    private _container: interfaces.Container;

    public getContainer(): interfaces.Container {
        return this._container;
    }

    public withIocContainer(): FrameworkBootstrapper {
        this._container = new Container({defaultScope: "Singleton"});
        this._container.load(new FrameworkModule());
        return this;
    }

    public withLogging(logLevel: LogLevel, pipelines?: Array<ILoggingPipeline>): FrameworkBootstrapper {
        if (this._container == undefined) {
            throw new Error("withIocContainer must be called before this method");
        }
        this._container.load(new LoggingModule(logLevel, pipelines));
        return this;
    }

    public bootstrap(): boolean {
        return true;
    }

}