"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
const Path = require("path");
const inversify_1 = require("inversify");
const NxConfiguration_1 = require("./NxConfiguration");
const PersistenceService_1 = require("../persistence/PersistenceService");
const ConfigurationFileNotFoundError_1 = require("../errors/ConfigurationFileNotFoundError");
const NxConfiguration_2 = require("./NxConfiguration");
const NxConfigurationFileName = "nx.json";
let ConfigurationService = class ConfigurationService {
    constructor(persistenceService) {
        this._persistenceService = persistenceService;
    }
    init() {
        return new Promise((resolve) => {
            let configuration = new NxConfiguration_1.NxConfiguration();
            configuration.link = new NxConfiguration_2.LinkScope();
            configuration.bump = new NxConfiguration_2.LinkScope();
            resolve(configuration);
        });
    }
    load(configurationFilePath) {
        return this._persistenceService.fileExists(configurationFilePath)
            .then((exists) => {
            if (exists === false) {
                throw new ConfigurationFileNotFoundError_1.ConfigurationFileNotFoundError(configurationFilePath);
            }
            return this._persistenceService.readFile(configurationFilePath)
                .then((data) => {
                let configuration = JSON.parse(data.toString("utf-8"));
                if (configuration.link != undefined && configuration.link.files != undefined) {
                    configuration.link.files = this.resolveDirectories(configuration.link.files, Path.dirname(configurationFilePath));
                }
                if (configuration.bump != undefined && configuration.bump.files != undefined) {
                    configuration.bump.files = this.resolveDirectories(configuration.bump.files, Path.dirname(configurationFilePath));
                }
                return configuration;
            });
        });
    }
    search(directory) {
        let directoryInfo = this._persistenceService.getDirectoryInfo(directory, false);
        if (directoryInfo.files != undefined) {
            for (let i = 0; i < directoryInfo.files.length; i++) {
                let file = directoryInfo.files[i];
                if (file.fileName === NxConfigurationFileName) {
                    return file.path;
                }
            }
        }
        return this.search(Path.dirname(directory));
    }
    save(directory, configuration) {
        let fullPath = Path.join(directory, NxConfigurationFileName);
        return this._persistenceService.fileExists(fullPath)
            .then((exists) => {
            if (exists === true) {
                return this._persistenceService.deleteFile(fullPath).then(() => {
                    return this._persistenceService.writeFile(fullPath, JSON.stringify(configuration, null, 4));
                });
            }
            else {
                return this._persistenceService.writeFile(fullPath, JSON.stringify(configuration, null, 4));
            }
        });
    }
    resolveDirectories(directories, currentDirectory) {
        return directories.map((dir) => {
            if (Path.isAbsolute(dir)) {
                return dir;
            }
            else {
                return Path.resolve(Path.join(currentDirectory, dir));
            }
        });
    }
};
ConfigurationService = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject("PersistenceService")),
    __metadata("design:paramtypes", [PersistenceService_1.PersistenceService])
], ConfigurationService);
exports.ConfigurationService = ConfigurationService;
//# sourceMappingURL=ConfigurationService.js.map