"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
const inversify_1 = require("inversify");
const Path = require("path");
const PersistenceService_1 = require("../persistence/PersistenceService");
const PackageService_1 = require("../node/PackageService");
const LoggingServiceFactory_1 = require("../logging/LoggingServiceFactory");
const NpmService_1 = require("../node/NpmService");
let LinkService = class LinkService {
    constructor(loggingServiceFactory, persistenceService, npmService, packageService) {
        this._loggingService = loggingServiceFactory.create("LinkService");
        this._persistenceService = persistenceService;
        this._npmService = npmService;
        this._packageService = packageService;
    }
    link(scope) {
        this._loggingService.debug("start linking");
        let packageInfos = this._packageService.locatePackages(scope.files);
        this._loggingService.debug("packages located");
        packageInfos.forEach((packageInfo) => {
            this._loggingService.debug(`located ${packageInfo.name} (v. ${packageInfo.version}) at ${packageInfo.manifestPath}`, packageInfo);
            packageInfo.dependencies.forEach((dependency) => {
                this._loggingService.debug(`checking dependency ${dependency.name}`, dependency);
                let packageToLink = packageInfos.find((x) => x.name === dependency.name);
                if (packageToLink != undefined) {
                    this._loggingService.debug(`dependency ${dependency.name} matched to package ${packageToLink.name}`, {
                        dependency: dependency,
                        packageToLink: packageToLink
                    });
                    this._npmService.link(packageInfo, packageToLink);
                    this._loggingService.info(`linked ${packageToLink.name} from ${packageInfo.name}`);
                }
                else {
                    this._loggingService.debug(`no package matched for dependency ${dependency.name}`, dependency);
                }
            });
        });
        return Promise.resolve(true);
    }
    unlink(scope) {
        this._loggingService.debug("start unlinking");
        let packageInfos = this._packageService.locatePackages(scope.files);
        this._loggingService.debug("packages located");
        packageInfos.forEach((packageInfo) => {
            this._loggingService.debug(`located ${packageInfo.name} (v. ${packageInfo.version}) at ${packageInfo.manifestPath}`, packageInfo);
            packageInfo.dependencies.forEach((dependency) => {
                this._loggingService.debug(`checking dependency ${dependency.name}`, dependency);
                let packageToUnlink = packageInfos.find((x) => x.name === dependency.name);
                if (packageToUnlink != undefined) {
                    this._loggingService.debug(`dependency ${dependency.name} matched to package ${packageToUnlink.name}`, {
                        dependency: dependency,
                        packageToLink: packageToUnlink
                    });
                    this.unlinkPackage(packageInfo, packageToUnlink);
                    this._loggingService.info(`unlinked ${packageToUnlink.name} from ${packageInfo.name}`);
                }
                else {
                    this._loggingService.debug(`no package matched for dependency ${dependency.name}`, dependency);
                }
            });
        });
        return Promise.resolve(true);
    }
    unlinkPackage(packageInfo, packageToUnlink) {
        let nodeModulesDirectoryPath = Path.join(Path.dirname(packageInfo.manifestPath), "node_modules");
        let nodeModulesDirectory = this._persistenceService.getDirectoryInfo(nodeModulesDirectoryPath, false);
        if (nodeModulesDirectory == undefined || nodeModulesDirectory.symbolicLinks == undefined) {
            this._loggingService.warn(`node modules folder not found ${nodeModulesDirectoryPath}`, nodeModulesDirectoryPath);
            return;
        }
        nodeModulesDirectory.symbolicLinks.forEach((moduleDirectory) => {
            let manifestPath = Path.join(moduleDirectory.path, "package.json");
            let manifest = this._packageService.loadManifest(manifestPath);
            if (manifest != undefined) {
                this._loggingService.warn(`manifest of ${manifest.name} loaded from ${manifestPath}`, manifest);
                if (packageToUnlink.name === manifest.name) {
                    this._persistenceService.unlinkDir(moduleDirectory.path);
                    this._loggingService.debug(`unlinked ${manifest.name} from ${packageInfo.name}`);
                }
            }
            else {
                this._loggingService.warn(`could not load manifest from ${manifestPath}`, manifestPath);
            }
        });
    }
};
LinkService = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject("LoggingServiceFactory")),
    __param(1, inversify_1.inject("PersistenceService")),
    __param(2, inversify_1.inject("NpmService")),
    __param(3, inversify_1.inject("PackageService")),
    __metadata("design:paramtypes", [LoggingServiceFactory_1.LoggingServiceFactory,
        PersistenceService_1.PersistenceService,
        NpmService_1.NpmService,
        PackageService_1.PackageService])
], LinkService);
exports.LinkService = LinkService;
//# sourceMappingURL=LinkService.js.map