import "reflect-metadata";
import {ContainerModule, interfaces} from "inversify";
import Bind = interfaces.Bind;
import {LoggingServiceFactory} from "./LoggingServiceFactory";
import {LogLevel} from "./LogLevel";
import {ILoggingPipeline} from "./ILoggingPipeline";

export class LoggingModule extends ContainerModule {

    constructor(logLevel: LogLevel, pipelines: Array<ILoggingPipeline>) {
        super((bind: Bind) => {
            LoggingModule.initializeBindings(bind, logLevel, pipelines);
        });
    }

    private static initializeBindings(bind: Bind, logLevel: LogLevel, pipelines: Array<ILoggingPipeline>): void {
        LoggingModule.initializeLoggingBindings(bind, logLevel, pipelines);
    }

    private static initializeLoggingBindings(bind: Bind, logLevel: LogLevel, pipelines: Array<ILoggingPipeline>): void {
        let loggerFactory = new LoggingServiceFactory(logLevel, pipelines);
        bind<LoggingServiceFactory>("LoggingServiceFactory").toConstantValue(loggerFactory);
    }
}