import {LogLevel} from "./LogLevel";
import {ILoggingPipeline} from "./ILoggingPipeline";

export class LoggingService {
    private _location: string;
    private _logLevel: LogLevel;
    private _pipelines: Array<ILoggingPipeline>;

    constructor(location: string, logLevel: LogLevel, pipelines: Array<ILoggingPipeline>) {
        this._location = location;
        this._logLevel = logLevel;
        this._pipelines = pipelines;
    }

    public info(message: string, data?: any): void {
        this.log(LogLevel.Info, message, data);
    }

    public warn(message: string, data?: any): void {
        this.log(LogLevel.Warn, message, data);
    }

    public error(message: string, data?: any): void {
        this.log(LogLevel.Error, message, data);
    }

    public debug(message: string, data?: any): void {
        this.log(LogLevel.Debug, message, data);
    }

    private log(logLevel: LogLevel, message: string, data: any): void {
        if (this._pipelines == undefined) {
            return;
        }
        this._pipelines.forEach((pipeline) => {
            pipeline.log(logLevel, this._location, message, data);
        });
    }
}