import {injectable} from "inversify";
import {LoggingService} from "./LoggingService";
import {LogLevel} from "./LogLevel";
import {ILoggingPipeline} from "./ILoggingPipeline";

export interface ILoggingServiceFactory {
    create(location: string): LoggingService;
}

@injectable()
export class LoggingServiceFactory implements ILoggingServiceFactory {

    private _logLevel: LogLevel;
    private _pipelines: Array<ILoggingPipeline>;

    constructor(logLevel: LogLevel, pipelines: Array<ILoggingPipeline>) {
        this._logLevel = logLevel;
        this._pipelines = pipelines;
    }

    public create(location: string): LoggingService {
        return new LoggingService(location, this._logLevel, this._pipelines);
    }

}