import {inject, injectable} from "inversify";
import * as ChildProcess from "child_process";
import * as Path from "path";
import * as Os from "os";
import {PackageInfo} from "./PackageInfo";
import {LoggingService} from "../logging/LoggingService";
import {LoggingServiceFactory} from "../logging/LoggingServiceFactory";


@injectable()
export class NpmService {

    private _loggingService: LoggingService;

    constructor(@inject("LoggingServiceFactory") loggingServiceFactory: LoggingServiceFactory) {
        this._loggingService = loggingServiceFactory.create("NpmService");
    }

    public link(packageInfo: PackageInfo, packageToLink: PackageInfo): Promise<void> {
        let cwd = Path.dirname(packageInfo.manifestPath);
        return this.startProcess(cwd, ["link", Path.dirname(packageToLink.manifestPath)])
            .then(() => {
                this._loggingService.debug(`successfully linked package ${packageToLink.name} for ${packageInfo.manifestPath}`, { packageInfo: packageInfo, packageToLink: packageToLink });
            })
            .catch((error) => {
                this._loggingService.error(`error while linking package ${packageToLink.name} for ${packageInfo.manifestPath}`, { packageInfo: packageInfo, packageToLink: packageToLink });
                throw error;
            });
    }

    private startProcess(cwd: string, args: Array<string>): Promise<boolean> {
        let commandName = Os.platform() === "win32" ? "npm.cmd" : "npm";
        let spawnOptions: ChildProcess.SpawnSyncOptions = {
            cwd: cwd,
        };
        let result = ChildProcess.spawnSync(commandName, args, spawnOptions);
        if (result.error != undefined) {
            return Promise.reject(result.error);
        }
        else {
            return Promise.resolve(true);
        }

    }
}