"use strict";
class PackageInfo {
    get name() {
        return this._name;
    }
    get manifestPath() {
        return this._manifestPath;
    }
    get version() {
        return this._version;
    }
    get dependencies() {
        return this._dependencies;
    }
    get devDependencies() {
        return this._devDependencies;
    }
    constructor(name, manifestPath, version, depenencies, devDependencies) {
        this._name = name;
        this._manifestPath = manifestPath;
        this._version = version;
        this._dependencies = this.getDependencies(depenencies);
        this._devDependencies = this.getDependencies(devDependencies);
    }
    getDependencies(depenencies) {
        let dependenciesArray = new Array();
        if (depenencies == undefined) {
            return dependenciesArray;
        }
        if (Array.isArray(depenencies)) {
            depenencies.forEach((x) => {
                dependenciesArray.push(new PackageDependency(x.name, x.version));
            });
        }
        else {
            for (let key in depenencies) {
                dependenciesArray.push(new PackageDependency(key, depenencies[key]));
            }
        }
        return dependenciesArray;
    }
    hasDependency(name) {
        if (this.dependencies == undefined) {
            return false;
        }
        return this.dependencies.find((x) => x.name === name) != undefined;
    }
    hasDevDependency(name) {
        if (this.devDependencies == undefined) {
            return false;
        }
        return this.devDependencies.find((x) => x.name === name) != undefined;
    }
}
exports.PackageInfo = PackageInfo;
class PackageDependency {
    get name() {
        return this._name;
    }
    get version() {
        return this._version;
    }
    constructor(name, version) {
        this._name = name;
        this._version = version;
    }
}
exports.PackageDependency = PackageDependency;
//# sourceMappingURL=PackageInfo.js.map