export class PackageInfo {

    private _name: string;
    private _manifestPath: string;
    private _version: string;
    private _dependencies: Array<PackageDependency>;
    private _devDependencies: Array<PackageDependency>;

    get name(): string {
        return this._name;
    }

    get manifestPath(): string {
        return this._manifestPath;
    }

    get version(): string {
        return this._version;
    }

    get dependencies(): Array<PackageDependency> {
        return this._dependencies;
    }

    get devDependencies(): Array<PackageDependency> {
        return this._devDependencies;
    }

    constructor(name: string, manifestPath: string, version: string, depenencies: Array<PackageDependency>, devDependencies: Array<PackageDependency>);
    constructor(name: string, manifestPath: string, version: string, depenencies: { [key: string]: string }, devDependencies: { [key: string]: string });
    constructor(name: string, manifestPath: string, version: string, depenencies: { [key: string]: string } | Array<PackageDependency>, devDependencies: { [key: string]: string } |  Array<PackageDependency>) {
        this._name = name;
        this._manifestPath = manifestPath;
        this._version = version;
        this._dependencies = this.getDependencies(depenencies);
        this._devDependencies = this.getDependencies(devDependencies);
    }

    private getDependencies(depenencies: { [key: string]: string } | Array<PackageDependency>): Array<PackageDependency> {
        let dependenciesArray = new Array<PackageDependency>();
        if (depenencies == undefined) {
            return dependenciesArray;
        }
        if (Array.isArray(depenencies)) {
            depenencies.forEach((x) => {
                dependenciesArray.push(new PackageDependency(x.name, x.version));
            });
        }
        else {
            for (let key in depenencies) {
                dependenciesArray.push(new PackageDependency(key, depenencies[key]));
            }
        }
        return dependenciesArray;
    }


    public hasDependency(name: string): boolean {
        if(this.dependencies == undefined){
            return false;
        }
        return this.dependencies.find((x) => x.name === name) != undefined;
    }

    public hasDevDependency(name: string): boolean {
        if(this.devDependencies == undefined){
            return false;
        }
        return this.devDependencies.find((x) => x.name === name) != undefined;
    }
}

export class PackageDependency {

    private _name: string;
    private _version: string;

    public get name(): string {
        return this._name;
    }

    public get version(): string {
        return this._version;
    }

    constructor(name: string, version: string) {
        this._name = name;
        this._version = version;
    }

}