"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
const inversify_1 = require("inversify");
const PackageInfo_1 = require("./PackageInfo");
const PersistenceService_1 = require("../persistence/PersistenceService");
const LoggingServiceFactory_1 = require("../logging/LoggingServiceFactory");
let PackageService = class PackageService {
    constructor(loggingServiceFactory, persistenceService) {
        this._loggingService = loggingServiceFactory.create("PackageService");
        this._persistenceService = persistenceService;
    }
    locatePackages(searchDirectories) {
        let packageFileInfos = this.getPackageFileInfos(searchDirectories);
        let packageInfos = packageFileInfos.map((packageFileInfo) => {
            let manifest = this.loadManifest(packageFileInfo.path);
            let packageInfo = new PackageInfo_1.PackageInfo(manifest.name, packageFileInfo.path, manifest.version, manifest.dependencies, manifest.devDependencies);
            return packageInfo;
        });
        this._loggingService.debug(`created package infos (${packageInfos.length})`, packageInfos);
        return packageInfos;
    }
    loadManifest(manifestPath) {
        this._loggingService.debug(`loading package manifest from ${manifestPath}`, manifestPath);
        let manifest = null;
        try {
            manifest = require(manifestPath);
        }
        catch (error) {
            this._loggingService.error(`error while loading manifest from ${manifestPath}`, manifestPath);
            throw error;
        }
        this._loggingService.debug(`manifest loaded (${manifest.name})`, manifest);
        return manifest;
    }
    getPackageFileInfos(searchDirectories) {
        this._loggingService.debug(`searching for packages in ${searchDirectories.join()}`, searchDirectories);
        let directoryInfos = searchDirectories.map((x) => {
            return this._persistenceService.getDirectoryInfo(x, true, ["node_modules", ".git", ".idea"]);
        });
        this._loggingService.debug("retrieved directory infos", directoryInfos);
        let jsonFiles = new Array();
        directoryInfos.forEach((x) => {
            let filteredFiles = this._persistenceService.getFileInfos(x, ".json");
            jsonFiles = jsonFiles.concat(filteredFiles);
        });
        this._loggingService.debug("filtered json files", jsonFiles);
        let packageFiles = jsonFiles.filter((x) => x.name === "package");
        this._loggingService.debug("filtered packages", packageFiles);
        return packageFiles;
    }
};
PackageService = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject("LoggingServiceFactory")),
    __param(1, inversify_1.inject("PersistenceService")),
    __metadata("design:paramtypes", [LoggingServiceFactory_1.LoggingServiceFactory,
        PersistenceService_1.PersistenceService])
], PackageService);
exports.PackageService = PackageService;
//# sourceMappingURL=PackageService.js.map