/// <reference types="node" />
export declare class PersistenceService {
    ensureDirectory(directoryPath: string): Promise<boolean>;
    getScopeDirectoryPath(): string;
    getNodevolutionDirectoryPath(): string;
    getFileInfos(directoryInfo: DirectoryInfo): Array<FileInfo>;
    getFileInfos(directoryInfo: DirectoryInfo, filter: string): Array<FileInfo>;
    getDirectoryInfo(directory: string, recursive: boolean): DirectoryInfo;
    getDirectoryInfo(directory: string, recursive: boolean, exclude: Array<string>): DirectoryInfo;
    private getRelevantWalkEntries(entries, exclude?);
    fileExists(path: string): Promise<boolean>;
    readFile(path: string): Promise<Buffer>;
    deleteFile(path: string): Promise<boolean>;
    unlinkDir(path: string): Promise<boolean>;
    writeFile(path: string, content: string): Promise<boolean>;
}
export declare class InfoBase {
    private _path;
    private _directory;
    private _name;
    readonly path: string;
    readonly directory: string;
    readonly name: string;
    constructor(path: string);
}
export declare class SymbolicLink extends InfoBase {
}
export declare class DirectoryInfo extends InfoBase {
    private _files;
    private _directories;
    private _symbolicLinks;
    readonly files: Array<FileInfo>;
    readonly directories: Array<DirectoryInfo>;
    readonly symbolicLinks: Array<SymbolicLink>;
    constructor(path: string, directories: Array<DirectoryInfo>, files: Array<FileInfo>, symbolicLinks: Array<SymbolicLink>);
}
export declare class FileInfo extends InfoBase {
    private _extension;
    private _fileName;
    readonly extension: string;
    readonly fileName: string;
    constructor(path: string);
}
