"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
const Fs = require("fs");
const Path = require("path");
const Os = require("os");
const inversify_1 = require("inversify");
let PersistenceService = class PersistenceService {
    ensureDirectory(directoryPath) {
        return new Promise((resolve, reject) => {
            Fs.exists(directoryPath, (exists) => {
                if (exists) {
                    resolve(true);
                }
                else {
                    Fs.mkdir(directoryPath, "0o777", (error) => {
                        if (error != undefined) {
                            reject(false);
                        }
                        else {
                            resolve(true);
                        }
                    });
                }
            });
        });
    }
    getScopeDirectoryPath() {
        return Path.join(this.getNodevolutionDirectoryPath(), "scopes");
    }
    getNodevolutionDirectoryPath() {
        let homeDirectoryPath = Os.homedir();
        return Path.join(homeDirectoryPath, ".nodevolution");
    }
    getFileInfos(directoryInfo, filter) {
        if (directoryInfo.files == undefined && directoryInfo.directories == undefined) {
            return undefined;
        }
        let fileInfos = null;
        if (filter == undefined) {
            fileInfos = directoryInfo.files;
        }
        else if (directoryInfo.files != undefined) {
            fileInfos = directoryInfo.files.filter((x) => {
                return x.extension === filter;
            });
        }
        if (directoryInfo.directories != undefined) {
            directoryInfo.directories.forEach((subDirectory) => {
                let subFiles = this.getFileInfos(subDirectory, filter);
                if (subFiles != undefined) {
                    if (fileInfos == undefined) {
                        fileInfos = subFiles;
                    }
                    else {
                        fileInfos = fileInfos.concat(subFiles);
                    }
                }
            });
        }
        return fileInfos;
    }
    getDirectoryInfo(directory, recursive, exclude) {
        let rootFullPath = Path.resolve(directory);
        let entries = Fs.readdirSync(rootFullPath);
        let fileInfos = null;
        let directoryInfos = null;
        let symbolicLinks = null;
        let relevantEntries = this.getRelevantWalkEntries(entries, exclude);
        relevantEntries.forEach((entry) => {
            let fullPath = Path.join(rootFullPath, entry);
            let stats = Fs.lstatSync(fullPath);
            if (stats.isFile()) {
                if (fileInfos == undefined) {
                    fileInfos = new Array();
                }
                fileInfos.push(new FileInfo(fullPath));
            }
            else if (stats.isDirectory()) {
                let directoryInfo = null;
                if (recursive) {
                    directoryInfo = this.getDirectoryInfo(fullPath, recursive, exclude);
                }
                else {
                    directoryInfo = new DirectoryInfo(fullPath, null, null, null);
                }
                if (directoryInfos == undefined) {
                    directoryInfos = new Array();
                }
                directoryInfos.push(directoryInfo);
            }
            else if (stats.isSymbolicLink()) {
                if (symbolicLinks == undefined) {
                    symbolicLinks = new Array();
                }
                symbolicLinks.push(new SymbolicLink(fullPath));
            }
        });
        return new DirectoryInfo(rootFullPath, directoryInfos, fileInfos, symbolicLinks);
    }
    getRelevantWalkEntries(entries, exclude) {
        if (exclude == undefined) {
            return entries;
        }
        else {
            return entries.filter((x) => {
                let segments = Path.parse(x);
                return exclude.indexOf(segments.base) === -1 && exclude.indexOf(x) === -1;
            });
        }
    }
    fileExists(path) {
        return new Promise((resolve, reject) => {
            Fs.access(path, (error) => {
                if (error != undefined) {
                    if (error.code === "EEXIST") {
                        resolve(true);
                    }
                    else {
                        resolve(false);
                    }
                }
                else {
                    resolve(true);
                }
            });
        });
    }
    readFile(path) {
        return new Promise((resolve, reject) => {
            Fs.readFile(path, (error, data) => {
                if (error != undefined) {
                    reject(error);
                }
                else {
                    resolve(data);
                }
            });
        });
    }
    deleteFile(path) {
        return new Promise((resolve, reject) => {
            Fs.unlink(path, (error) => {
                if (error != undefined) {
                    reject(error);
                }
                else {
                    resolve(true);
                }
            });
        });
    }
    unlinkDir(path) {
        return new Promise((resolve, reject) => {
            Fs.unlink(path, (error) => {
                if (error != undefined) {
                    reject(error);
                }
                else {
                    resolve(true);
                }
            });
        });
    }
    writeFile(path, content) {
        return new Promise((resolve, reject) => {
            Fs.writeFile(path, content, { encoding: "utf-8" }, (error) => {
                if (error != undefined) {
                    reject(error);
                }
                else {
                    resolve(true);
                }
            });
        });
    }
};
PersistenceService = __decorate([
    inversify_1.injectable(),
    __metadata("design:paramtypes", [])
], PersistenceService);
exports.PersistenceService = PersistenceService;
class InfoBase {
    get path() {
        return this._path;
    }
    get directory() {
        return this._directory;
    }
    get name() {
        return this._name;
    }
    constructor(path) {
        this._path = path;
        let segments = Path.parse(path);
        this._directory = segments.dir;
        this._name = segments.name;
    }
}
exports.InfoBase = InfoBase;
class SymbolicLink extends InfoBase {
}
exports.SymbolicLink = SymbolicLink;
class DirectoryInfo extends InfoBase {
    get files() {
        return this._files;
    }
    get directories() {
        return this._directories;
    }
    get symbolicLinks() {
        return this._symbolicLinks;
    }
    constructor(path, directories, files, symbolicLinks) {
        super(path);
        this._directories = directories;
        this._files = files;
        this._symbolicLinks = symbolicLinks;
    }
}
exports.DirectoryInfo = DirectoryInfo;
class FileInfo extends InfoBase {
    get extension() {
        return this._extension;
    }
    get fileName() {
        return this._fileName;
    }
    constructor(path) {
        super(path);
        let segments = Path.parse(path);
        this._extension = segments.ext;
        this._fileName = `${segments.name}${segments.ext}`;
    }
}
exports.FileInfo = FileInfo;
//# sourceMappingURL=PersistenceService.js.map