import { LoggingServiceFactory } from "../logging/LoggingServiceFactory";
import { PackageInfo } from "../node/PackageInfo";
import { VersionLevel } from "./VersionLevel";
import { PersistenceService } from "../persistence/PersistenceService";
export declare class VersionService {
    private _loggingService;
    private _bumpCommandFactory;
    private _persistenceService;
    constructor(loggingServiceFactory: LoggingServiceFactory, persistenceService: PersistenceService);
    createVersinBumpInfos(initialPackage: PackageInfo, versionLevel: VersionLevel, packages: Array<PackageInfo>): Array<BumpInfo>;
    private createBumpInfos(commands);
    private sortByDependencies(that, other);
    persistChanges(bumpInfos: Array<BumpInfo>): Promise<void>;
    private persistBumpInfoChanges(bumpInfo);
    private buildBumpCommands(treeItem, versionLevel, commands);
    private buildTreeItem(packageInfo, packages);
    private getBumpedVersion(version, level);
    getVersionLevel(versionLevel: string): VersionLevel;
    private getSemverVersionLevel(level);
}
export declare class BumpInfo {
    readonly commands: Array<BumpCommandBase>;
    readonly packageName: string;
    readonly packageInfo: PackageInfo;
    private _commands;
    private _packageInfo;
    constructor(packageInfo: PackageInfo, commands: Array<BumpCommandBase>);
}
export declare class BumpCommandFactory {
    createBumpPackageCommand(packageInfo: PackageInfo, version: string, newVersion: string): BumpPackageCommand;
    createBumpDependencyCommand(packageInfo: PackageInfo, dependencyPackageName: string, version: string, newVersion: string): BumpDependencyCommand;
    createBumpDevDependencyCommand(packageInfo: PackageInfo, dependencyPackageName: string, version: string, newVersion: string): BumpDevDependencyCommand;
}
export declare class DependencyTreeItem {
    readonly dependendItems: Array<DependencyTreeItem>;
    readonly packageName: string;
    readonly packageInfo: PackageInfo;
    private _dependendItems;
    private _packageInfo;
    constructor(packageInfo: PackageInfo);
    add(item: DependencyTreeItem): void;
}
export declare abstract class BumpCommandBase {
    private _version;
    private _newVersion;
    private _packageInfo;
    readonly packageName: string;
    readonly version: string;
    readonly newVersion: string;
    readonly packageInfo: PackageInfo;
    constructor(packageInfo: PackageInfo, version: string, newVersion: string);
}
export declare class BumpPackageCommand extends BumpCommandBase {
}
export declare class BumpDependencyCommand extends BumpCommandBase {
    private _dependencyPackageName;
    readonly dependencyPackageName: string;
    constructor(packageInfo: PackageInfo, dependencyPackageName: string, version: string, newVersion: string);
}
export declare class BumpDevDependencyCommand extends BumpCommandBase {
    private _dependencyPackageName;
    readonly dependencyPackageName: string;
    constructor(packageInfo: PackageInfo, dependencyPackageName: string, version: string, newVersion: string);
}
