"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const inversify_1 = require("inversify");
const semver = require("semver");
const LoggingServiceFactory_1 = require("../logging/LoggingServiceFactory");
const InvalidVersionError_1 = require("../errors/InvalidVersionError");
const VersionLevel_1 = require("./VersionLevel");
const PersistenceService_1 = require("../persistence/PersistenceService");
let VersionService = class VersionService {
    constructor(loggingServiceFactory, persistenceService) {
        this._persistenceService = persistenceService;
        this._loggingService = loggingServiceFactory.create("LinkService");
        this._bumpCommandFactory = new BumpCommandFactory();
    }
    createVersinBumpInfos(initialPackage, versionLevel, packages) {
        let tree = this.buildTreeItem(initialPackage, packages);
        let commands = Array();
        this.buildBumpCommands(tree, versionLevel, commands);
        return this.createBumpInfos(commands);
    }
    createBumpInfos(commands) {
        let bumpPackageCommands = commands.filter((x) => x instanceof BumpPackageCommand);
        let infos = bumpPackageCommands.map((x) => {
            let groupedByPackage = new Array();
            groupedByPackage = groupedByPackage.concat(commands.filter((y) => y.packageName === x.packageName));
            return new BumpInfo(x.packageInfo, groupedByPackage);
        });
        return infos.sort(this.sortByDependencies);
    }
    sortByDependencies(that, other) {
        if (that.commands.length > other.commands.length) {
            return 1;
        }
        if (that.commands.length < other.commands.length) {
            return -1;
        }
        return 0;
    }
    persistChanges(bumpInfos) {
        return __awaiter(this, void 0, void 0, function* () {
            let persistencePromises = bumpInfos.map((bumpInfo) => {
                if (bumpInfo.commands != undefined && bumpInfo.commands.length > 0) {
                    return this.persistBumpInfoChanges(bumpInfo);
                }
                else {
                    return Promise.resolve();
                }
            });
            return Promise.all(persistencePromises)
                .then(() => {
                return;
            });
        });
    }
    persistBumpInfoChanges(bumpInfo) {
        return __awaiter(this, void 0, void 0, function* () {
            let content = (yield this._persistenceService.readFile(bumpInfo.packageInfo.manifestPath)).toString();
            let packageFile = JSON.parse(content);
            bumpInfo.commands.forEach((cmd) => {
                if (cmd instanceof BumpPackageCommand) {
                    packageFile.version = cmd.newVersion;
                }
                else if (cmd instanceof BumpDependencyCommand) {
                    packageFile.dependencies[cmd.dependencyPackageName] = cmd.newVersion;
                }
                else if (cmd instanceof BumpDevDependencyCommand) {
                    packageFile.devDependencies[cmd.dependencyPackageName] = cmd.newVersion;
                }
            });
            yield this._persistenceService.writeFile(bumpInfo.packageInfo.manifestPath, JSON.stringify(packageFile, null, "\t"));
        });
    }
    buildBumpCommands(treeItem, versionLevel, commands) {
        let isNotInCommands = commands.find((x) => {
            return x instanceof BumpPackageCommand && x.packageName === treeItem.packageName;
        }) == undefined;
        if (isNotInCommands) {
            let bumpedVersion = this.getBumpedVersion(treeItem.packageInfo.version, versionLevel);
            commands.push(this._bumpCommandFactory.createBumpPackageCommand(treeItem.packageInfo, treeItem.packageInfo.version, bumpedVersion));
            if (treeItem.dependendItems != undefined) {
                treeItem.dependendItems.forEach((x) => {
                    this.buildBumpCommands(x, versionLevel, commands);
                    if (x.packageInfo.hasDependency(treeItem.packageName)) {
                        commands.push(this._bumpCommandFactory.createBumpDependencyCommand(x.packageInfo, treeItem.packageName, treeItem.packageInfo.version, bumpedVersion));
                    }
                    if (x.packageInfo.hasDevDependency(treeItem.packageName)) {
                        commands.push(this._bumpCommandFactory.createBumpDevDependencyCommand(x.packageInfo, treeItem.packageName, treeItem.packageInfo.version, bumpedVersion));
                    }
                });
            }
        }
    }
    buildTreeItem(packageInfo, packages) {
        let item = new DependencyTreeItem(packageInfo);
        packages.forEach((p) => {
            if (p.hasDependency(packageInfo.name)) {
                item.add(this.buildTreeItem(p, packages));
            }
        });
        return item;
    }
    getBumpedVersion(version, level) {
        let validVersion = semver.valid(version);
        if (validVersion == undefined) {
            let error = new InvalidVersionError_1.InvalidVersionError(version);
            this._loggingService.error(`${error.message} - ${version}`, error);
            throw error;
        }
        this._loggingService.debug(`passed version is a valid semver version - ${validVersion}`, validVersion);
        let semverVersionLevel = this.getSemverVersionLevel(level);
        let incrementedVersion = semver.inc(validVersion, semverVersionLevel);
        this._loggingService.debug(`incremented version ${validVersion} => ${incrementedVersion}`);
        return incrementedVersion;
    }
    getVersionLevel(versionLevel) {
        let loweredVersionLevel = versionLevel.toLowerCase();
        switch (loweredVersionLevel) {
            case "major":
                return VersionLevel_1.VersionLevel.Major;
            case "minor":
                return VersionLevel_1.VersionLevel.Minor;
            case "patch":
                return VersionLevel_1.VersionLevel.Patch;
            case "premajor":
                return VersionLevel_1.VersionLevel.Premajor;
            case "preminor":
                return VersionLevel_1.VersionLevel.Preminor;
            case "prepatch":
                return VersionLevel_1.VersionLevel.Prepatch;
            case "prerelease":
                return VersionLevel_1.VersionLevel.Prerelease;
            default:
                return undefined;
        }
    }
    getSemverVersionLevel(level) {
        switch (level) {
            case VersionLevel_1.VersionLevel.Major:
                return "major";
            case VersionLevel_1.VersionLevel.Minor:
                return "minor";
            case VersionLevel_1.VersionLevel.Patch:
                return "patch";
            case VersionLevel_1.VersionLevel.Premajor:
                return "premajor";
            case VersionLevel_1.VersionLevel.Preminor:
                return "preminor";
            case VersionLevel_1.VersionLevel.Prepatch:
                return "prepatch";
            case VersionLevel_1.VersionLevel.Prerelease:
                return "prerelease";
        }
    }
};
VersionService = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject("LoggingServiceFactory")),
    __param(1, inversify_1.inject("PersistenceService")),
    __metadata("design:paramtypes", [LoggingServiceFactory_1.LoggingServiceFactory,
        PersistenceService_1.PersistenceService])
], VersionService);
exports.VersionService = VersionService;
class BumpInfo {
    get commands() {
        return this._commands;
    }
    get packageName() {
        return this._packageInfo.name;
    }
    get packageInfo() {
        return this._packageInfo;
    }
    constructor(packageInfo, commands) {
        this._packageInfo = packageInfo;
        this._commands = commands;
    }
}
exports.BumpInfo = BumpInfo;
class BumpCommandFactory {
    createBumpPackageCommand(packageInfo, version, newVersion) {
        return new BumpPackageCommand(packageInfo, version, newVersion);
    }
    createBumpDependencyCommand(packageInfo, dependencyPackageName, version, newVersion) {
        return new BumpDependencyCommand(packageInfo, dependencyPackageName, version, newVersion);
    }
    createBumpDevDependencyCommand(packageInfo, dependencyPackageName, version, newVersion) {
        return new BumpDevDependencyCommand(packageInfo, dependencyPackageName, version, newVersion);
    }
}
exports.BumpCommandFactory = BumpCommandFactory;
class DependencyTreeItem {
    get dependendItems() {
        return this._dependendItems;
    }
    get packageName() {
        return this._packageInfo.name;
    }
    get packageInfo() {
        return this._packageInfo;
    }
    constructor(packageInfo) {
        this._packageInfo = packageInfo;
    }
    add(item) {
        if (this._dependendItems == undefined) {
            this._dependendItems = new Array();
        }
        this._dependendItems.push(item);
    }
}
exports.DependencyTreeItem = DependencyTreeItem;
class BumpCommandBase {
    get packageName() {
        return this._packageInfo.name;
    }
    get version() {
        return this._version;
    }
    get newVersion() {
        return this._newVersion;
    }
    get packageInfo() {
        return this._packageInfo;
    }
    constructor(packageInfo, version, newVersion) {
        this._packageInfo = packageInfo;
        this._version = version;
        this._newVersion = newVersion;
    }
}
exports.BumpCommandBase = BumpCommandBase;
class BumpPackageCommand extends BumpCommandBase {
}
exports.BumpPackageCommand = BumpPackageCommand;
class BumpDependencyCommand extends BumpCommandBase {
    get dependencyPackageName() {
        return this._dependencyPackageName;
    }
    constructor(packageInfo, dependencyPackageName, version, newVersion) {
        super(packageInfo, version, newVersion);
        this._dependencyPackageName = dependencyPackageName;
    }
}
exports.BumpDependencyCommand = BumpDependencyCommand;
class BumpDevDependencyCommand extends BumpCommandBase {
    get dependencyPackageName() {
        return this._dependencyPackageName;
    }
    constructor(packageInfo, dependencyPackageName, version, newVersion) {
        super(packageInfo, version, newVersion);
        this._dependencyPackageName = dependencyPackageName;
    }
}
exports.BumpDevDependencyCommand = BumpDevDependencyCommand;
//# sourceMappingURL=VersionService.js.map